@@ -146,6 +146,8 @@ import Foreign.ForeignPtr
 import Foreign.Ptr
 import Foreign.Marshal.Array ( advancePtr, copyArray )
 
+import Control.DeepSeq ( NFData )
+
 import Control.Monad.ST ( ST )
 import Control.Monad.Primitive
 
@@ -178,6 +180,8 @@ data Vector a = Vector {-# UNPACK #-} !Int
                        {-# UNPACK #-} !(ForeignPtr a)
         deriving ( Typeable )
 
+instance NFData (Vector a)
+
 instance (Show a, Storable a) => Show (Vector a) where
   showsPrec = G.showsPrec
 
@@ -590,7 +594,7 @@ generateM = G.generateM
 -- | Execute the monadic action and freeze the resulting vector.
 --
 -- @
--- create (do { v \<- new 2; write v 0 \'a\'; write v 1 \'b\' }) = \<'a','b'\>
+-- create (do { v \<- new 2; write v 0 \'a\'; write v 1 \'b\'; return v }) = \<'a','b'\>
 -- @
 create :: Storable a => (forall s. ST s (MVector s a)) -> Vector a
 {-# INLINE create #-}
